@@ -3,7 +3,7 @@ package Tk::ColorEntry;
 use strict;
 use warnings;
 use vars qw($VERSION);
-$VERSION = '0.09';
+$VERSION = '0.11';
 use Tk;
 
 use base qw(Tk::Derived Tk::Frame);
@@ -33,6 +33,8 @@ cancels the pick operation instead.
 
 =head1 OPTIONS
 
+You can use many options of L<Tk::ColorPicker>.
+
 =over 4
 
 =item Switch: B<-command>
@@ -68,6 +70,8 @@ Reference to the variable where the current value is held.
 
 =head1 METHODS
 
+You can use many methods of L<Tk::ColorPicker>.
+
 =over 4
 
 =cut
@@ -100,11 +104,12 @@ sub Populate {
 		},
 		-widget => $self,
 	) unless defined $pop;
+	$self->Advertise('Pop', $pop);
 
 	$entry->bind('<Button-1>', [$self, 'popBlock']);
 	$entry->bind('<ButtonRelease-1>', [$self, 'popFlip']);
 	$entry->bind('<Return>', [$self, 'popFlip']);
-	$entry->bind('<FocusOut>', [$self, 'popDown']);
+#	$entry->bind('<FocusOut>', [$self, 'popDown']);
 	$entry->bind('<Key>', [$self, 'OnKey']);
 	$entry->bind('<Escape>', [$self, 'OnEscape']);
 
@@ -117,6 +122,7 @@ sub Populate {
 		-entryforeground => ['PASSIVE', undef, undef, $self->Subwidget('Entry')->cget('-foreground')],
 		-font => [$entry],
 		-foreground => [$entry],
+		-historyfile => [$pop->Subwidget('Picker')],
 		-indborderwidth => [{-borderwidth => $indicator}, undef, undef, 2],
 		-indicatorwidth => [{-width => $indicator}, undef, undef, 4],
 		-indrelief => [{-relief => $indicator}, undef, undef, 'sunken'],
@@ -141,7 +147,9 @@ sub EntryUpdate {
 	my $display = $self->Subwidget('Display');
 	my $val = $entry->get;
 	if ($self->validate($val)) {
-		$display->configure(-background => $val);
+		my $pop = $self->Subwidget('Pop');
+		my $current = $pop->getHEX;	
+		$display->configure(-background => $current) if $self->validate($current);
 		$entry->configure(-foreground => $self->cget('-entryforeground'));
 	} else {
 		$display->configure(-background => $self->cget('-background'));
@@ -239,15 +247,11 @@ sub put {
 	}
 	my $var = $self->Subwidget('Entry')->cget('-textvariable');
 	$$var = $color;
+	my $pop = $self->Subwidget('Pop');
+	$pop->put($color);
 	$self->EntryUpdate;
 }
 
-sub validate {
-	my ($self, $val) = @_;
-	my $repeat = $self->cget('-popcolor')->colordepth / 4;
-	return $val =~ /^#(?:[0-9a-fA-F]{3}){$repeat}$/
-}
-
 =back
 
 =head1 LICENSE
